# -*- coding: utf-8 -*-
"""
Test map index setting.

"""
import starry
import numpy as np
import pytest


@pytest.fixture(scope="class", params=[(None,), (3,)])
def map(request):
    (nw,) = request.param
    map = starry.Map(ydeg=5, udeg=2, nw=nw)
    return map


class TestGettersAndSetters:
    """Test the Map getters and setters."""

    def test_ylm_single_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1, 1, 0] = 7
            assert map[1, 1, 0] == 7
            assert map.y[3, 0] == 7
        else:
            map[1, 1] = 7
            assert map[1, 1] == 7
            assert map.y[3] == 7

    def test_ylm_multiple_l_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1:, 1, 0] = 7
            assert np.all(map[:, 1, 0] == 7)
            inds = [l ** 2 + l + 1 for l in range(1, map.ydeg + 1)]
            assert np.all(map.y[inds, 0] == 7)
        else:
            map[1:, 1] = 7
            assert np.all(map[:, 1] == 7)
            inds = [l ** 2 + l + 1 for l in range(1, map.ydeg + 1)]
            assert np.all(map.y[inds] == 7)

    def test_ylm_multiple_m_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1, :, 0] = 7
            assert np.all(map[1, :, 0] == 7)
            inds = [1, 2, 3]
            assert np.all(map.y[inds, 0] == 7)
        else:
            map[1, :] = 7
            assert np.all(map[1, :] == 7)
            inds = [1, 2, 3]
            assert np.all(map.y[inds] == 7)

    def test_ylm_multiple_x_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1, 1, :] = 7
            assert np.all(map[1, 1, :] == 7)
            assert np.all(map.y[3, :] == 7)
        else:
            pass

    def test_ylm_multiple_lm_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1:, :, 0] = 7
            assert np.all(map[1:, :, 0] == 7)
            assert np.all(map.y[1:, 0] == 7)
        else:
            map[1:, :] = 7
            assert np.all(map[1:, :] == 7)
            assert np.all(map.y[1:] == 7)

    def test_ylm_multiple_lx_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1:, 1, :] = 7
            assert np.all(map[1:, 1, :] == 7)
            inds = [l ** 2 + l + 1 for l in range(1, map.ydeg + 1)]
            assert np.all(map.y[inds, :] == 7)
        else:
            pass

    def test_ylm_multiple_mx_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1, :, :] = 7
            assert np.all(map[1, :, :] == 7)
            inds = [1, 2, 3]
            assert np.all(map.y[inds, :] == 7)
        else:
            pass

    def test_ylm_multiple_lmx_to_scalar(self, map):
        map.reset()
        if map.nw:
            map[1:, :, :] = 7
            assert np.all(map[1:, :, :] == 7)
            assert np.all(map.y[1:, :] == 7)
        else:
            pass

    def test_ylm_multiple_l_to_vector(self, map):
        map.reset()
        if map.nw:
            map[1:, 1, 0] = np.atleast_2d(np.arange(1, map.ydeg + 1)).T
            assert np.allclose(map[:, 1, 0].flatten(), range(1, map.ydeg + 1))
            inds = [l ** 2 + l + 1 for l in range(1, map.ydeg + 1)]
            assert np.allclose(map.y[inds, 0], range(1, map.ydeg + 1))
        else:
            map[1:, 1] = range(1, map.ydeg + 1)
            assert np.allclose(map[:, 1], range(1, map.ydeg + 1))
            inds = [l ** 2 + l + 1 for l in range(1, map.ydeg + 1)]
            assert np.allclose(map.y[inds], range(1, map.ydeg + 1))

    def test_ylm_multiple_m_to_vector(self, map):
        map.reset()
        if map.nw:
            map[1, :, 0] = np.array([[1], [2], [3]])
            inds = [1, 2, 3]
            assert np.allclose(map[1, :, 0].flatten(), inds)
            assert np.allclose(map.y[inds, 0], inds)
        else:
            map[1, :] = [1, 2, 3]
            inds = [1, 2, 3]
            assert np.allclose(map[1, :], inds)
            assert np.allclose(map.y[inds], inds)

    def test_ylm_multiple_w_to_vector(self, map):
        map.reset()
        if map.nw:
            map[1, 1, :] = [1, 2, 3]
            assert np.allclose(map[1, 1, :], [1, 2, 3])
            assert np.allclose(map.y[3, :], [1, 2, 3])
        else:
            pass

    def test_ylm_multiple_lm_to_vector(self, map):
        map.reset()
        if map.nw:
            map[1:, :, 0] = np.atleast_2d(np.arange(1, map.Ny)).T
            assert np.allclose(map[1:, :, 0].flatten(), np.arange(1, map.Ny))
            assert np.allclose(map.y[1:, 0], np.arange(1, map.Ny))
        else:
            map[1:, :] = np.arange(1, map.Ny)
            assert np.allclose(map[1:, :], np.arange(1, map.Ny))
            assert np.allclose(map.y[1:], np.arange(1, map.Ny))

    def test_ylm_multiple_lx_to_vector(self, map):
        map.reset()
        if map.nw:
            vals = np.array(
                [np.arange(1, map.ydeg + 1) + i for i in range(map.nw)]
            ).transpose()
            map[1:, 1, :] = vals
            assert np.allclose(map[1:, 1, :], vals)
            inds = [l ** 2 + l + 1 for l in range(1, map.ydeg + 1)]
            assert np.allclose(map.y[inds, :], vals)
        else:
            pass

    def test_ylm_multiple_mx_to_vector(self, map):
        map.reset()
        if map.nw:
            vals = np.array(
                [[1 + i, 2 + i, 3 + i] for i in range(map.nw)]
            ).transpose()
            map[1, :, :] = vals
            assert np.allclose(map[1, :, :], vals)
            inds = [1, 2, 3]
            assert np.allclose(map.y[inds, :], vals)
        else:
            pass

    def test_ylm_multiple_lmx_to_vector(self, map):
        map.reset()
        if map.nw:
            np.random.seed(43)
            vals = np.random.randn(map.Ny - 1, map.nw)
            map[1:, :, :] = vals
            assert np.allclose(map[1:, :, :], vals)
            assert np.allclose(map.y[1:, :], vals)
        else:
            pass

    def test_ul_single_to_scalar(self, map):
        map.reset()
        map[1] = 7
        assert map[1] == 7
        assert map.u[1] == 7

    def test_ul_multiple_to_scalar(self, map):
        map.reset()
        map[1:] = 7
        assert np.all(map[1:] == 7)
        assert np.all(map.u[1:] == 7)

    def test_ul_multiple_to_vector(self, map):
        map.reset()
        map[1:] = [1, 2]
        assert np.allclose(map[1:], [1, 2])
        assert np.allclose(map.u[1:], [1, 2])

    def test_yl_set_constant_coeff(self, map):
        map.reset()
        if map.nw:
            try:
                map[0, 0, :] = 2.0
                raise Exception("")
            except ValueError:
                pass
        else:
            try:
                map[0, 0] = 2.0
                raise Exception("")
            except ValueError:
                pass

    def test_ul_set_constant_coeff(self, map):
        map.reset()
        try:
            map[0] = 0.0
            raise Exception("")
        except ValueError:
            pass

    def test_ul_constant_coeff(self, map):
        map.reset()
        assert np.allclose(map[:], [-1, 0, 0])

    def test_set_all(self, map):
        map.reset()
        if not map.nw:
            map[:, :] = 1
            assert np.allclose(map[:, :], np.ones(map.Ny))

    def test_set_transposed(self, map):
        map.reset()
        if map.nw:
            vals = np.ones((map.nw, map.Ny - 1))
            map[1:, :, :] = vals
            assert np.allclose(map[1:, :, :], vals.T)



# -*- coding: utf-8 -*-
"""Test theano compiling."""
import starry
import pytest
from starry.compat import theano
import numpy as np


def test_test_value_error():
    """
    Ensure that no error is raised when there
    are missing test values in our compiled
    functions, as pymc3 forces theano errors when
    test values are missing.

    """
    theano.config.compute_test_value = "raise"
    map = starry.Map()
    flux = map.flux()
    assert theano.config.compute_test_value == "raise"



# -*- coding: utf-8 -*-
from ...compat import Apply, Op, tt
import numpy as np


__all__ = ["sTOp", "rTReflectedOp", "sTReflectedOp"]


class sTOp(Op):
    def __init__(self, func, N):
        self.func = func
        self.N = N
        self._grad_op = sTGradientOp(self)

    def make_node(self, *inputs):
        inputs = [tt.as_tensor_variable(i) for i in inputs]
        outputs = [tt.TensorType(inputs[-1].dtype, (False, False))()]
        return Apply(self, inputs, outputs)

    def infer_shape(self, *args):
        shapes = args[-1]
        return [shapes[0] + (tt.as_tensor(self.N),)]

    def R_op(self, inputs, eval_points):
        if eval_points[0] is None:
            return eval_points
        return self.grad(inputs, eval_points)

    def perform(self, node, inputs, outputs):
        outputs[0][0] = self.func(*inputs)

    def grad(self, inputs, gradients):
        return self._grad_op(*(inputs + gradients))


class sTGradientOp(Op):
    def __init__(self, base_op):
        self.base_op = base_op

    def make_node(self, *inputs):
        inputs = [tt.as_tensor_variable(i) for i in inputs]
        outputs = [i.type() for i in inputs[:-1]]
        return Apply(self, inputs, outputs)

    def infer_shape(self, *args):
        shapes = args[-1]
        return shapes[:-1]

    def perform(self, node, inputs, outputs):
        bb, br = self.base_op.func(*inputs)
        outputs[0][0] = np.reshape(bb, np.shape(inputs[0]))
        outputs[1][0] = np.reshape(br, np.shape(inputs[1]))


class rTReflectedOp(Op):
    def __init__(self, func, N):
        self.func = func
        self.N = N
        self._grad_op = rTReflectedGradientOp(self)

    def make_node(self, *inputs):
        inputs = [tt.as_tensor_variable(i) for i in inputs]
        outputs = [
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
        ]
        return Apply(self, inputs, outputs)

    def infer_shape(self, *args):
        shapes = args[-1]
        return [
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
        ]

    def R_op(self, inputs, eval_points):
        if eval_points[0] is None:
            return eval_points
        return self.grad(inputs, eval_points)

    def perform(self, node, inputs, outputs):
        (b, sigr) = inputs
        rT, ddb, ddsigr = self.func(b, sigr)
        outputs[0][0] = rT
        outputs[1][0] = ddb
        outputs[2][0] = ddsigr

    def grad(self, inputs, gradients):
        results = self(*inputs)
        grad = self._grad_op(*(inputs + results + [gradients[0]]))
        return grad


class rTReflectedGradientOp(Op):
    def __init__(self, base_op):
        self.base_op = base_op

    def make_node(self, *inputs):
        inputs = [tt.as_tensor_variable(i) for i in inputs]
        outputs = [i.type() for i in inputs[:2]]
        return Apply(self, inputs, outputs)

    def infer_shape(self, *args):
        shapes = args[-1]
        return shapes[:2]

    def perform(self, node, inputs, outputs):
        b, sigr, rT, ddb, ddsigr, brT = inputs
        bb = (brT * ddb).sum(-1)
        bsigr = (brT * ddsigr).sum()
        outputs[0][0] = np.reshape(bb, np.shape(b))
        outputs[1][0] = np.array(np.reshape(bsigr, np.shape(sigr)))


class sTReflectedOp(Op):
    def __init__(self, func, N):
        self.func = func
        self.N = N
        self._grad_op = sTReflectedGradientOp(self)

    def make_node(self, *inputs):
        inputs = [tt.as_tensor_variable(i) for i in inputs]
        outputs = [
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
            tt.TensorType(inputs[-1].dtype, (False, False))(),
        ]
        return Apply(self, inputs, outputs)

    def infer_shape(self, *args):
        shapes = args[-1]
        return [
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
            shapes[0] + (tt.as_tensor(self.N),),
        ]

    def R_op(self, inputs, eval_points):
        if eval_points[0] is None:
            return eval_points
        return self.grad(inputs, eval_points)

    def perform(self, node, inputs, outputs):
        b, theta, bo, ro, sigr = inputs
        sT, ddb, ddtheta, ddbo, ddro, ddsigr = self.func(
            b, theta, bo, ro, sigr
        )
        outputs[0][0] = sT
        outputs[1][0] = ddb
        outputs[2][0] = ddtheta
        outputs[3][0] = ddbo
        outputs[4][0] = ddro
        outputs[5][0] = ddsigr

    def grad(self, inputs, gradients):
        results = self(*inputs)
        grad = self._grad_op(*(inputs + results + [gradients[0]]))
        return grad


class sTReflectedGradientOp(Op):
    def __init__(self, base_op):
        self.base_op = base_op

    def make_node(self, *inputs):
        inputs = [tt.as_tensor_variable(i) for i in inputs]
        outputs = [i.type() for i in inputs[:5]]
        return Apply(self, inputs, outputs)

    def infer_shape(self, *args):
        shapes = args[-1]
        return shapes[:5]

    def perform(self, node, inputs, outputs):
        (
            b,
            theta,
            bo,
            ro,
            sigr,
            sT,
            ddb,
            ddtheta,
            ddbo,
            ddro,
            ddsigr,
            bsT,
        ) = inputs
        bb = (bsT * ddb).sum(-1)
        btheta = (bsT * ddtheta).sum(-1)
        bbo = (bsT * ddbo).sum(-1)
        bro = (bsT * ddro).sum()
        bsigr = (bsT * ddsigr).sum()
        outputs[0][0] = np.reshape(bb, np.shape(b))
        outputs[1][0] = np.reshape(btheta, np.shape(theta))
        outputs[2][0] = np.reshape(bbo, np.shape(bo))
        outputs[3][0] = np.array(np.reshape(bro, np.shape(ro)))
        outputs[4][0] = np.array(np.reshape(bsigr, np.shape(sigr)))



# -*- coding: utf-8 -*-
from .starry_version import __version__


# Store the package directory
import os

_PACKAGEDIR = os.path.abspath(os.path.dirname(__file__))


# Theano compatibility hacks
from . import compat


# Import the main interface
from ._config import config
from . import kepler, linalg, maps
from .maps import Map
from .kepler import Primary, Secondary, System


# Clean up the namespace
del os


__all__ = [
    "__version__",
    "kepler",
    "linalg",
    "maps",
    "Map",
    "Primary",
    "Secondary",
    "System",
]



import numpy as np
import starry
import matplotlib.pyplot as plt
from scipy.interpolate import interp1d
from tqdm import tqdm
from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import pytest


@pytest.mark.parametrize(
    "xs,ys,zs,ro",
    [
        [1.0, 2.0, -1.0, 0.6],
        [1.0, 2.0, -1.0, 5.0],
        [1.0, 2.0, -1.0, 5.0],
        [1.0, 2.0, -1.0, 50.0],
        [0.0, 2.0, -1.0, 0.4],
        [0.0, -1.0, -1.0, 0.4],
        [0.0, -1.0, -1.0, 0.4],
        [1.0, 0.0, -1.0, 0.4],
        [1.0, 0.0, -1.0, 0.1],
        [1.0, 0.0, -1.0, 0.8],
        [1.0, 0.0, 0.0, 0.8],
    ],
)
def test_edges(
    xs, ys, zs, ro, y=[1, 1, 1], ns=100, nb=50, res=999, atol=1e-2, plot=False
):

    # Instantiate
    ydeg = np.sqrt(len(y) + 1) - 1
    map = starry.Map(ydeg=ydeg, reflected=True)
    map[1:, :] = y

    # bo - ro singularities
    singularities = [ro - 1, 0, ro, 1, 1 - ro, 1 + ro]
    labels = [
        "$b_o = r_o - 1$",
        "$b_o = 0$",
        "$b_o = r_o$",
        "$b_o = 1$",
        "$b_o = 1 - r_o$",
        "$b_o = 1 + r_o$",
        "grazing",
        "grazing",
    ]

    # Find where the occultor grazes the terminator
    rs = np.sqrt(xs ** 2 + ys ** 2 + zs ** 2)
    b = -zs / rs
    theta = -np.arctan2(xs, ys)
    tol = 1e-15
    nx = 10
    c = np.cos(theta)
    s = np.sin(theta)
    t = np.tan(theta)
    q2 = c ** 2 + b ** 2 * s ** 2

    # Bottom / top half of occultor
    for sgn0 in [1, -1]:

        # Successively refine x array
        xest = 0
        xdel = ro
        for j in range(10):
            x = np.linspace(xest - xdel, xest + xdel, nx)

            # Divide & conquer
            yomax = 1 + ro
            yomin = -1 - ro
            niter = 0
            xest = 0
            while niter < 100 and np.abs(yomax - yomin) > tol:
                yo_ = 0.5 * (yomax + yomin)
                y = yo_ + sgn0 * np.sqrt(ro ** 2 - x ** 2)
                try:

                    # Scan the x axis for an intersection
                    for i in range(nx):

                        # There are two solutions to the quadratic; pick
                        # the one that's actually on the ellipse
                        p = (x[i] * c - b * s * np.sqrt(q2 - x[i] ** 2)) / q2
                        yt1 = p * s + b * np.sqrt(1 - p ** 2) * c
                        xr = x[i] * c + yt1 * s
                        yr = -x[i] * s + yt1 * c
                        arg1 = np.abs(xr ** 2 + (yr / b) ** 2 - 1)
                        p = (x[i] * c + b * s * np.sqrt(q2 - x[i] ** 2)) / q2
                        yt2 = p * s + b * np.sqrt(1 - p ** 2) * c
                        xr = x[i] * c + yt2 * s
                        yr = -x[i] * s + yt2 * c
                        arg2 = np.abs(xr ** 2 + (yr / b) ** 2 - 1)
                        if arg1 < arg2:
                            if arg1 < 1e-6:
                                yt = yt1
                            else:
                                continue
                        elif arg2 < arg1:
                            if arg2 < 1e-6:
                                yt = yt2
                            else:
                                continue
                        else:
                            continue

                        if (sgn0 == -1) and (y[i] < yt):
                            # Part of the occultor has dipped below the terminator
                            yomin = yo_
                            xest = x[i]
                            raise StopIteration

                        if (sgn0 == 1) and (y[i] > yt):
                            # Part of the occultor has dipped above the terminator
                            yomax = yo_
                            xest = x[i]
                            raise StopIteration

                except StopIteration:
                    niter += 1
                    continue
                else:
                    niter += 1
                    if sgn0 == -1:
                        # The occultor is above the terminator everywhere
                        yomax = yo_
                    else:
                        # The occultor is below the terminator everywhere
                        yomin = yo_

            # Increase res by 10x
            xdel /= 10

        singularities.append(yo_)

    # Arrays over singularities
    yo_s = np.zeros((8, ns))
    logdelta = np.append(-np.inf, np.linspace(-16, -2, ns // 2 - 1))
    delta = np.concatenate((-(10 ** logdelta[::-1]), 10 ** logdelta))
    for i, pt in enumerate(singularities):
        yo_s[i] = pt + delta
    yo_s = yo_s[np.argsort(singularities)]
    labels = list(np.array(labels)[np.argsort(singularities)])

    # Array over full occultation
    yo_full = np.linspace(yo_s[0, 0], yo_s[-1, -1], ns, endpoint=True)

    # All
    yo = np.concatenate((yo_full.reshape(1, -1), yo_s))

    # Compute analytic
    flux = np.zeros_like(yo)
    msg = [["" for n in range(yo.shape[1])] for m in range(yo.shape[0])]
    for i in range(len(yo)):
        for k in tqdm(range(ns)):
            try:
                flux[i, k] = map.flux(
                    xs=xs, ys=ys, zs=zs, xo=0, yo=yo[i, k], ro=ro
                )
            except Exception as e:
                flux[i, k] = 0.0
                msg[i][k] = str(e).split("\n")[0]

    # Compute numerical
    flux_num = np.zeros_like(yo) * np.nan
    flux_num_interp = np.zeros_like(yo) * np.nan
    (lat, lon), (x, y, z) = map.ops.compute_ortho_grid(res)
    img = map.render(xs=xs, ys=ys, zs=zs, res=res).flatten()
    for i in range(len(yo)):
        for k in tqdm(range(ns)):
            idx = x ** 2 + (y - yo[i, k]) ** 2 > ro ** 2
            flux_num_interp[i, k] = np.nansum(img[idx]) * 4 / res ** 2
            if (k == 0) or (k == ns - 1) or (k % (ns // nb) == 0):
                flux_num[i, k] = flux_num_interp[i, k]

        # Adjust the baseline
        offset = np.nanmedian(flux[i]) - np.nanmedian(flux_num_interp[i])
        flux_num_interp[i] += offset
        flux_num[i] += offset

    # Plot
    if plot:

        fig = plt.figure(figsize=(10, 8))
        fig.subplots_adjust(hspace=0.35)
        ax = [
            plt.subplot2grid((40, 40), (0, 0), rowspan=15, colspan=40),
            plt.subplot2grid((40, 40), (20, 0), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (20, 10), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (20, 20), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (20, 30), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (30, 0), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (30, 10), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (30, 20), rowspan=10, colspan=10),
            plt.subplot2grid((40, 40), (30, 30), rowspan=10, colspan=10),
        ]

        # Prepare image for plotting
        img[(img < 0) | (img > 0.0)] = 1
        img = img.reshape(res, res)
        cmap = plt.get_cmap("plasma")
        cmap.set_under("grey")

        # Full light curve
        ax[0].plot(yo[0], flux[0], "k-", lw=1)
        ax[0].plot(yo[0], flux_num[0], "k.", lw=1)
        ax[0].tick_params(labelsize=10)
        ax[0].set_xlabel("$b_o$")
        ax[0].set_ylabel("flux")

        # Each singularity
        for i in range(1, len(yo)):
            ax[0].plot(yo[i], flux[i], lw=3, color="C{}".format(i - 1))
            ax[i].plot(
                2 + logdelta,
                flux[i][: ns // 2],
                lw=2,
                color="C{}".format(i - 1),
            )
            ax[i].plot(
                -(2 + logdelta)[::-1],
                flux[i][ns // 2 :],
                lw=2,
                color="C{}".format(i - 1),
            )
            ax[i].plot(2 + logdelta, flux_num[i][: ns // 2], "k.", ms=2)
            ax[i].plot(
                -(2 + logdelta)[::-1], flux_num[i][ns // 2 :], "k.", ms=2
            )
            ax[i].set_xticks([])
            ax[i].set_yticks([])

            # Show the map
            axins = inset_axes(
                ax[i], width="30%", height="30%", loc=4, borderpad=1
            )
            axins.imshow(
                img,
                origin="lower",
                cmap=cmap,
                extent=(-1, 1, -1, 1),
                vmin=1e-8,
            )
            circ = plt.Circle(
                (0, yo[i, ns // 2]),
                ro,
                fc="k",
                ec="k",
                clip_on=(ro > 0.75),
                zorder=99,
            )
            axins.add_artist(circ)
            axins.annotate(
                labels[i - 1],
                xy=(0.5, -0.1),
                xycoords="axes fraction",
                clip_on=False,
                ha="center",
                va="top",
                fontsize=8,
            )
            axins.set_xlim(-1.01, 1.01)
            axins.set_ylim(-1.01, 1.01)
            axins.axis("off")

        plt.show()

    # Compare
    if not np.allclose(flux, flux_num_interp, atol=atol):
        index = np.unravel_index(
            np.argmax(np.abs(flux - flux_num_interp)), flux.shape
        )
        if index[0] > 0:
            raise ValueError(
                "Error in singular region {}/8: {}".format(
                    index[0], labels[index[0] - 1]
                )
            )



# -*- coding: utf-8 -*-
"""
Unit unit tests.

"""
import starry
import astropy.units as u
from astropy.constants import G
import numpy as np
import pytest


def test_default_body_units():
    body = starry.Primary(starry.Map())
    assert body.length_unit == u.Rsun
    assert body.mass_unit == u.Msun
    assert body.angle_unit == u.degree
    assert body.time_unit == u.day
    assert body.map.angle_unit == u.degree


def test_default_system_units():
    pri = starry.Primary(starry.Map())
    sec = starry.Secondary(starry.Map(), porb=1.0)
    sys = starry.System(pri, sec)
    assert sys.time_unit == u.day


def test_unit_conversions():
    body = starry.Secondary(
        starry.Map(ydeg=1, inc=1.0, obl=1.0, angle_unit=u.radian),
        r=1.0,
        m=1.0,
        inc=1.0,
        porb=1.0,
        prot=1.0,
        t0=1.0,
        theta0=1.0,
        Omega=1.0,
        length_unit=u.cm,
        mass_unit=u.gram,
        time_unit=u.second,
        angle_unit=u.radian,
    )
    assert body.r == 1.0
    assert body._r == body.length_unit.in_units(u.Rsun)
    assert body.m == 1.0
    assert body._m == body.mass_unit.in_units(u.Msun)
    assert body.inc == 1.0
    assert body._inc == body.angle_unit.in_units(u.radian)
    assert body.theta0 == 1.0
    assert body._theta0 == body.angle_unit.in_units(u.radian)
    assert body.Omega == 1.0
    assert body._Omega == body.angle_unit.in_units(u.radian)
    assert body.prot == 1.0
    assert body._prot == body.time_unit.in_units(u.day)
    assert body.t0 == 1.0
    assert body._t0 == body.time_unit.in_units(u.day)
    assert body.ecc == 0.0
    assert body.map.inc == 1.0
    assert body.map._inc == body.map.angle_unit.in_units(u.radian)
    assert body.map.obl == 1.0
    assert body.map._obl == body.map.angle_unit.in_units(u.radian)

    # First test the orbital period
    assert body.porb == 1.0
    assert body._porb == body.time_unit.in_units(u.day)

    # Now test the semi-major axis
    assert body.a is None
    body.a = 1.0
    assert body.a == 1.0
    assert body._a == body.length_unit.in_units(u.Rsun)
    assert body.porb is None


def test_omega_w():
    """Ensure `w` and `omega` are equivalent."""
    body = starry.Secondary(starry.Map(), porb=1.0, w=30.0)
    assert np.allclose(body.omega, 30)
    body.omega = 60
    assert np.allclose(body.w, 60)

    body = starry.Secondary(starry.Map(), porb=1.0, omega=30.0)
    assert np.allclose(body.w, 30)
    body.w = 60
    assert np.allclose(body.omega, 60)


def test_period_semi():
    # Check that an error is raised if neither a nor porb is given
    with pytest.raises(ValueError) as e:
        body = starry.Secondary(starry.Map())
    assert "Must provide a value for either `porb` or `a`" in str(e.value)

    # Check that the semi --> period conversion works
    pri = starry.Primary(starry.Map(), m=1.0, mass_unit=u.Msun)
    sec = starry.Secondary(
        starry.Map(), a=10.0, m=1.0, length_unit=u.AU, mass_unit=u.Mearth
    )
    sys = starry.System(pri, sec)
    period = sys._get_periods()[0]
    true_period = (
        (
            (2 * np.pi)
            * (sec.a * sec.length_unit) ** (3 / 2)
            / (np.sqrt(G * (pri.m * pri.mass_unit + sec.m * sec.mass_unit)))
        )
        .to(u.day)
        .value
    )
    assert np.allclose(period, true_period)
